from DatabasePane import DatabasePane
from ConstructorPane import ConstructorPane
from DiagramPane import DiagramPane
from MethodPane import MethodPane
from PyMethodPane import PyMethodPane
from PyModulePane import PyModulePane
